--########################################################################
--
--  Este arquivo contm as seguintes entidades :
--      ==> Decodificador para acesso  entrada/sada e/ou memria
--      ==> Memria na forma de um vetor, com o cdigo objeto em um package
--
--   FERNANDO GEHM MORAES   03/agosto/2005
--   NEY CALAZANS			15/abril/2006
--
--########################################################################

--########################################################################
--
-- Decodificador para acesso  entrada/sada ou memria
--
--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;
use work.cleo.all;

entity es_decoder is
    port(ce, rw:  in std_logic;
         address: in internal_bus;
         ceCPU, rwCPU, ceES, rwES: out std_logic);
end es_decoder;

architecture a1 of es_decoder is

  signal add_es  : std_logic;
  
begin
  
    -- 
    -- acesso a um dos endereos FD, FE e FF designa acesso  entrada e sada
    --
    add_es <= '1' when address=x"FF" or address=x"FE" or address=x"FD" else
              '0';
                
    ceCPU <= '1' when ce='1' and add_es='0'  else '0';
    ceES  <= '1' when ce='1' and add_es='1'  else '0';
    
    
    rwCPU <= '1' when rw='1' and add_es='0'  else '0';
    rwES  <= '1' when rw='1' and add_es='1'  else '0';
   
     
end a1;



--########################################################################
--
-- memria na forma de um vetor - cdigo objeto definido em package
--
--########################################################################
library IEEE;
use ieee.STD_LOGIC_UNSIGNED.all;
use ieee.std_logic_1164.all;
use work.cleo.all;				-- package com definicoes basicas
use work.object_code.all;		-- cdigo objeto definido neste package

entity RAM_mem is
      port( clock, ce, rw : in  std_logic;
            address:  in  internal_bus;
            datain:   in  internal_bus;
            dataout:  out internal_bus
			);
end RAM_mem;

architecture RAM_mem of RAM_mem is 
   signal memoria : ram := (
x"44",
x"4C",
x"54",
x"4E",
x"24",
x"50",
x"9C",
x"02",
x"84",
x"10",
x"44",
x"4F",
x"50",
x"01",
x"24",
x"4F",
x"44",
x"4B",
x"54",
x"4F",
x"24",
x"4F",
x"C4",
x"43",
x"44",
x"4D",
x"54",
x"4F",
x"24",
x"4F",
x"C4",
x"43",
x"44",
x"4F",
x"24",
x"FF",
x"44",
x"50",
x"24",
x"FE",
x"44",
x"FD",
x"24",
x"FD",
x"BC",
x"14",
x"44",
x"50",
x"50",
x"01",
x"24",
x"50",
x"94",
x"38",
x"8C",
x"E8",
x"44",
x"4F",
x"50",
x"01",
x"24",
x"4F",
x"C4",
x"43",
x"8C",
x"DE",
x"F0",
x"94",
x"46",
x"D0",
x"40",
x"01",
x"24",
x"51",
x"D0",
x"3F",
x"84",
x"3E",
x"CA",
x"00",
x"00",
x"00",
	others => x"00");

begin    

   process(clock)
   begin
       if clock'event and clock='1' then
          if ce='1' and rw='0' then               
			  memoria(CONV_INTEGER(address)) <= datain;
			  -- writes in memory
          end if;  
          
          dataout <=  memoria(CONV_INTEGER(address)) ;            
		  -- memory reading, this is the default.
		  -- note that there is no tristate in the memory bus
          
       end if;
   end process;    
   
end RAM_mem;