--########################################################################
--
--                               CIRCUITO TOP
--
--   FERNANDO GEHM MORAES		03/agosto/2005
--   NEY LAERT VILAR CALAZANS	16/abril/2006
--
--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_unsigned.all;   -- uso conv_integer
use IEEE.Std_Logic_arith.all;
 
entity top is
          port( ck :      	in  std_logic;
                reset:    	in  std_logic;
                entrada: 	in  std_logic_vector(7 downto 0);
                saidah:		out std_logic_vector(7 downto 0);
                saidal:		out std_logic_vector(7 downto 0)
             );
end top;
        

architecture top of top is
                                      
   signal halt, hold, ce, rw, ceCPU, rwCPU, ceES, rwES: std_logic;
   signal address, datainCPU, dataoutCPU, dataoutRAM : std_logic_vector(7 downto 0);
  
begin

   -------------------------------------------------------------------------------
   -- instancia o processador Clepatra
   --
   cpuC : entity work.cleopatra port map
               ( clock=>ck, reset=>reset, halt=>halt, hold=>hold,
                 ce=>ce, rw=>rw, address=>address ,datain=>datainCPU, dataout=>dataoutCPU);

   datainCPU <= dataoutRAM when ceCPU='1' else 
   				entrada    when ceES='1'  else    
	   			-- CPU coloca no acumulador o valor das chaves
                (others=>'Z');
   
   -------------------------------------------------------------------------------
   -- instancia a memria, j com o programa inicializado
   --
   ramC: entity work.RAM_mem port map
               ( clock=>ck, ce=>ceCPU,rw=>rwCPU, address=>address, datain=>dataoutCPU, dataout=>dataoutRAM);
 
 
   -------------------------------------------------------------------------------
   -- instancia o decodificador que define se  acesso  memria ou entrada/dada
   --
   deco: entity work.es_decoder port map
             ( ce => ce, rw=>rw, address => address,
               ceCPU=>ceCPU, rwCPU=>rwCPU, ceES=>ceES, rwES=>rwES);
 
 
   -------------------------------------------------------------------------------
   -- captura o dado da CPU quando  feito acesso  entrada/sada 
   --
   process (ck, reset)
    begin
         if reset='1' then
              saidah    <= (others=>'0');            
              saidal    <= (others=>'0');            
         elsif ck'event and ck='1' then
              if ceES='1' and  rwES='0' then 
			  if address=x"FF" then saidah <= dataoutCPU; end if;
			  if address=x"FE" then saidal <= dataoutCPU; end if;
				  -- envia para uma das saidas dado do processador
              end if;  
         end if;
   end process;
 
   --------------------------------------------------------------------------
   -- Hold is not active
   --
   hold <= '0';
      
end top;
